@@ -19,7 +19,7 @@ from __future__ import annotations
 
 from functools import wraps
 import re
-from typing import Callable, Dict, TYPE_CHECKING
+from typing import Callable, Dict, Optional, TYPE_CHECKING, Tuple, Type, TypeVar
 from urllib.parse import unquote
 
 import requests
@@ -33,6 +33,8 @@ if TYPE_CHECKING:
 
 ABSOLUTE_URL_PATTERN_RE = re.compile(r'^[\w\?]+://[^/].*')
 
+URLType = TypeVar('URLType', bound='URL')
+
 
 class UrlNotResolvable(Exception):
     """
@@ -47,11 +49,34 @@ class URL:
     It takes one or several regexps to match urls, and an optional Page
     class which is instancied by PagesBrowser.open if the page matches a regex.
 
+    .. warning::
+
+        The ``methods`` parameter is only used for page matching, not request
+        building using :py:meth:`URL.go` or :py:meth:`URL.open`; you must
+        still set the method using these.
+
     :param base: The name of the browser's property containing the base URL.
+    :param headers: Headers to include on requests using this URL.
+    :param timeout: Timeout to use for this URL in particular.
+    :param methods: Request HTTP methods to match the response.
+    :param content_type: MIME type of the content to match the response with.
     """
     _creation_counter = 0
 
-    def __init__(self, *args, base: str = 'BASEURL'):
+    def __init__(
+        self, *args,
+        base: str = 'BASEURL',
+        headers: Optional[Dict[str, str]] = None,
+        timeout: Optional[float] = None,
+        methods: Tuple[str, ...] = (),
+        content_type: Optional[str] = None,
+    ):
+        if content_type is not None and ';' in content_type:
+            raise ValueError(
+                'Content-Type matching is only based on the MIME type, '
+                + 'not additional properties such as encoding or version',
+            )
+
         self.urls = []
         self.klass = None
         self.browser = None
@@ -62,6 +87,10 @@ class URL:
                 self.klass = arg
 
         self._base = base
+        self._headers = headers
+        self._timeout = timeout
+        self._methods = tuple(methods)
+        self._content_type = content_type
         self._creation_counter = URL._creation_counter
         URL._creation_counter += 1
 
@@ -74,6 +103,9 @@ class URL:
         assert self.klass is not None, "You can use this method only if there is a Page class handler."
         assert self.browser is not None
 
+        if self.browser.page is None:
+            return False
+
         if len(kwargs):
             m = self.match(self.build(**kwargs))
             assert m is not None
@@ -81,15 +113,29 @@ class URL:
         else:
             params = None
 
+        if not isinstance(self.browser.page, self.klass):
+            return False
+
+        if self._methods:
+            method = self.browser.response.request.method
+            if method not in self._methods:
+                return False
+
+        if self._content_type is not None:
+            content_type = self.browser.response.headers.get('Content-Type')
+            if content_type is None:
+                return False
+
+            content_type, _, _ = content_type.partition(';')
+            content_type = content_type.strip()
+            if content_type != self._content_type:
+                return False
+
         # XXX use unquote on current params values because if there are spaces
         # or special characters in them, it is encoded only in but not in kwargs.
         return (
-            self.browser.page and
-            isinstance(self.browser.page, self.klass) and
-            (
-                params is None or
-                params == {k: unquote(v) for k, v in self.browser.page.params.items()}
-            )
+            params is None or
+            params == {k: unquote(v) for k, v in self.browser.page.params.items()}
         )
 
     def stay_or_go(
@@ -124,6 +170,7 @@ class URL:
         json: Dict | None = None,
         method: str | None = None,
         headers: Dict[str, str] |  None = None,
+        timeout: Optional[float] = None,
         **kwargs
     ) -> requests.Response | Page:
         """
@@ -136,8 +183,22 @@ class URL:
         """
         assert self.browser is not None
 
-        r = self.browser.location(self.build(**kwargs), params=params, data=data, json=json, method=method,
-                                  headers=headers or {})
+        headers = headers or {}
+        if self._headers:
+            headers.update(self._headers)
+
+        if timeout is None:
+            timeout = self._timeout
+
+        r = self.browser.location(
+            self.build(**kwargs),
+            params=params,
+            data=data,
+            json=json,
+            method=method,
+            headers=headers,
+            timeout=timeout,
+        )
         return r.page or r
 
     def open(
@@ -148,6 +209,7 @@ class URL:
         json: Dict | None = None,
         method: str | None = None,
         headers: Dict[str, str] | None = None,
+        timeout: float | None = None,
         is_async: bool = False,
         callback: Callable[[requests.Response], requests.Response] = lambda response: response,
         **kwargs
@@ -162,7 +224,23 @@ class URL:
         """
         assert self.browser is not None
 
-        r = self.browser.open(self.build(**kwargs), params=params, data=data, json=json, method=method, headers=headers or {}, is_async=is_async, callback=callback)
+        headers = headers or {}
+        if self._headers:
+            headers.update(self._headers)
+
+        if timeout is not None:
+            timeout = self._timeout
+
+        r = self.browser.open(
+            self.build(**kwargs),
+            params=params,
+            data=data,
+            json=json,
+            method=method,
+            headers=headers,
+            is_async=is_async,
+            callback=callback,
+        )
 
         if hasattr(r, 'page') and r.page:
             return r.page
@@ -272,6 +350,17 @@ class URL:
             return None
         if response.request.method == 'HEAD':
             return None
+        if self._methods and response.request.method not in self._methods:
+            return None
+        if self._content_type is not None:
+            content_type = response.headers.get('Content-Type')
+            if content_type is None:
+                return None
+
+            content_type, _, _ = content_type.partition(';')
+            content_type = content_type.strip()
+            if content_type != self._content_type:
+                return None
 
         m = self.match(response.url)
         if m:
@@ -313,30 +402,112 @@ class URL:
             return func(browser, id_or_url, *args, **kwargs)
         return inner
 
-    def with_page(self, cls: Page) -> URL:
+    def with_headers(
+        self: URLType,
+        headers: Optional[Dict[str, str]],
+    ) -> URLType:
+        """Get the current URL with different stored headers.
+
+        For example, suppose that a browser needs to add an 'Accept'
+        header for accessing a specific header of the API; see `Using the
+        Accept Header to version your API`_ for more details.
+
+        .. code-block:: python
+
+            class MyBrowser(PagesBrowser):
+                products = URL('products')
+
+            class MyChildBrowser(MyBrowser):
+                BASEURL = 'https://products-api.example/'
+
+                products = MyBrowser.products.with_headers({
+                    'Accept': 'application/vnd.example.api+json;version=2',
+                })
+
+        .. _`Using the Accept Header to version your API`:
+            https://labs.qandidate.com/blog/2014/10/16/using-the-accept-header
+            -to-version-your-api/
+
+        :param headers: The new headers to set to the URL.
+        :return: The URL using the different headers.
+        """
+        new_url = self.__class__(
+            *self.urls,
+            self.klass,
+            base=self._base,
+            headers=headers,
+            timeout=self._timeout,
+            methods=self._methods,
+            content_type=self._content_type,
+        )
+        new_url.browser = None
+        return new_url
+
+    def without_headers(self: URLType) -> URLType:
+        """Get the current URL without stored headers.
+
+        :return: The URL using the different headers.
+        """
+        return self.with_headers(None)
+
+    def with_timeout(self: URLType, timeout: Optional[float]) -> URLType:
+        """Get a new URL object with timeout.
+
+        :param timeout: The new timeout to apply, or ``None`` if the default
+            timeout from the browser is to be used.
+        :return: The URL using the different timeout.
+        """
+        new_url = self.__class__(
+            *self.urls,
+            self.klass,
+            base=self._base,
+            headers=self._headers,
+            timeout=timeout,
+            methods=self._methods,
+            content_type=self._content_type,
+        )
+        new_url.browser = None
+        return new_url
+
+    def without_timeout(self: URLType) -> URLType:
+        """Get a new URL object using the browser's timeout.
+
+        :return: The URL without the custom timeout.
+        """
+        return self.with_timeout(None)
+
+    def with_page(self: URLType, cls: Type[Page]) -> URLType:
         """Get a new URL with the same path but a different page class.
 
         :param cls: The new page class to use.
+        :return: The URL object with the updated page class.
         """
-        new_url = self.__class__(*self.urls, cls, base=self._base)
+        new_url = self.__class__(
+            *self.urls,
+            cls,
+            base=self._base,
+            headers=self._headers,
+            timeout=self._timeout,
+            methods=self._methods,
+            content_type=self._content_type,
+        )
         new_url.browser = None
         return new_url
 
     def with_urls(
-        self,
+        self: URLType,
         *urls: str,
         clear: bool = True,
         match_new_first: bool = True
-    ) -> URL:
+    ) -> URLType:
         """Get a new URL object with the same page but with different paths.
 
-        :param str urls: List of urls handled by the page
-        :param bool clear: If True, the page will only handled the given urls.
-                           Otherwise, the urls are added to already handled
-                           urls.
-        :param bool match_new_first: If true, new paths will be matched first
-                                     for this URL; this parameter is ignored
-                                     when `clear` is True.
+        :param urls: List of urls handled by the page.
+        :param clear: If True, the page will only handled the given urls.
+            Otherwise, the urls are added to already handled urls.
+        :param match_new_first: If true, new paths will be matched first
+            for this URL; this parameter is ignored when ``clear`` is True.
+        :return: The URL object with the updated patterns.
         """
         new_urls = list(urls)
         if not clear:
@@ -349,10 +520,86 @@ class URL:
         # We only want unique patterns here.
         new_urls = list(dict.fromkeys(new_urls))
 
-        new_url = self.__class__(*new_urls, self.klass, base=self._base)
+        new_url = self.__class__(
+            *new_urls,
+            self.klass,
+            base=self._base,
+            headers=self._headers,
+            timeout=self._timeout,
+            methods=self._methods,
+            content_type=self._content_type,
+        )
         new_url.browser = None
         return new_url
 
+    def with_base(
+        self: URLType,
+        base: str = 'BASEURL',
+    ) -> URLType:
+        """Get a new URL object with a custom base.
+
+        :param base: The name of the new base, or None to use the default one.
+        :return: The URL object with the updated base.
+        """
+        return self.__class__(
+            *self.urls,
+            self.klass,
+            base=base,
+            headers=self._headers,
+            timeout=self._timeout,
+            methods=self._methods,
+            content_type=self._content_type,
+        )
+
+    def with_methods(
+        self: URLType,
+        methods: Tuple[str, ...],
+    ) -> URLType:
+        """Get a new URL object with custom methods.
+
+        :param methods: The new methods to match the URL with.
+        :return: The URL object with the updated methods.
+        """
+        return self.__class__(
+            *self.urls,
+            self.klass,
+            base=self._base,
+            headers=self._headers,
+            timeout=self._timeout,
+            methods=methods,
+            content_type=self._content_type,
+        )
+
+    def without_methods(self: URLType) -> URLType:
+        """Get a new URL object without matching on methods.
+
+        :return: The URL object with the updated methods.
+        """
+        return self.with_methods(())
+
+    def with_content_type(self: URLType, content_type: Optional[str]) -> URLType:
+        """Get a new URL object with custom Content-Type matching.
+
+        :param content_type: The new content type to match with.
+        :return: The URL object with the updated content type to match.
+        """
+        return self.__class__(
+            *self.urls,
+            self.klass,
+            base=self._base,
+            headers=self._headers,
+            timeout=self._timeout,
+            methods=self._methods,
+            content_type=content_type,
+        )
+
+    def without_content_type(self: URLType) -> URLType:
+        """Get a new URL object without Content-Type matching.
+
+        :return: The URL object with no content type matching.
+        """
+        return self.with_content_type(None)
+
 
 class BrowserParamURL(URL):
     r"""A URL that automatically fills some params from browser attributes.
